"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _template = _interopRequireDefault(require("@babel/template"));

var _pluginSyntaxJsx = _interopRequireDefault(require("@babel/plugin-syntax-jsx"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var idGeneratorLibraryName = '@inline-svg-unique-id/react';
var idGeneratorHookName = 'useUniqueInlineId';
var idAttributeName = 'id'; // Matches SVG IRI: url(#example)

var iriRegex = 'url\\(#([a-zA-Z][\\w:.-]*)\\)';
var iriExactMatchRegex = new RegExp("^".concat(iriRegex, "$"));
var iriGlobalMatchRegex = new RegExp(iriRegex, 'g');
var svgIdentifiers = {
  svg: ['svg', 'Svg'],
  defs: ['defs', 'Defs'],
  style: ['style']
};

var isSvgComponentIdentifier = function isSvgComponentIdentifier(openingElementNameIdentifier, expectedSvgIdentifiers) {
  return expectedSvgIdentifiers.some(function (i) {
    return openingElementNameIdentifier.isJSXIdentifier({
      name: i
    });
  });
};

var isSvgComponentPath = function isSvgComponentPath(path, expectedSvgIdentifiers) {
  return isSvgComponentIdentifier(path.get('openingElement.name'), expectedSvgIdentifiers);
};

var isSvgPath = function isSvgPath(path) {
  return isSvgComponentPath(path, svgIdentifiers.svg);
};

var isDefsPath = function isDefsPath(path) {
  return isSvgComponentPath(path, svgIdentifiers.defs);
};

var isStylePath = function isStylePath(path) {
  return isSvgComponentPath(path, svgIdentifiers.style);
};

var isStringLiteralAttribute = function isStringLiteralAttribute(attribute) {
  return attribute.get('value').isStringLiteral();
};

var createIriUrl = function createIriUrl(id) {
  return "url(#".concat(id, ")");
};

var createIdValuesContainer = function createIdValuesContainer(_createIdIdentifier) {
  var idIdentifierByIdValueMap = new Map();
  return {
    createIdIdentifier: function createIdIdentifier(idValue) {
      if (idIdentifierByIdValueMap.has(idValue)) {
        return idIdentifierByIdValueMap.get(idValue);
      }

      var newIdentifier = _createIdIdentifier();

      idIdentifierByIdValueMap.set(idValue, newIdentifier);
      return newIdentifier;
    },
    getIdIdentifier: function getIdIdentifier(idValue) {
      return idIdentifierByIdValueMap.get(idValue);
    },
    hasIdentifiers: function hasIdentifiers() {
      return idIdentifierByIdValueMap.size !== 0;
    },
    getIdentifiers: function getIdentifiers() {
      return Array.from(idIdentifierByIdValueMap.values());
    }
  };
};

var buildIriUrlExpression = _template["default"].expression("`".concat(createIriUrl('${%%idIdentifier%%}'), "`"));

var buildIdGeneratorHookImportStatement = (0, _template["default"])("import { ".concat(idGeneratorHookName, " } from '").concat(idGeneratorLibraryName, "';"));
var buildIdIdentifierGeneratorStatement = (0, _template["default"])("const %%idIdentifier%% = ".concat(idGeneratorHookName, "();"));

var plugin = function plugin(_ref) {
  var t = _ref.types;

  var splitStylesStringByIriToLiterals = function splitStylesStringByIriToLiterals(stylesString, idValuesWithIdentifiers) {
    return idValuesWithIdentifiers.reduce(function (splitStylesLiterals, _ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          idValue = _ref3[0],
          idIdentifier = _ref3[1];

      return splitStylesLiterals.flatMap(function (stylesOrIdIdentifier) {
        if (typeof stylesOrIdIdentifier !== 'string') {
          return stylesOrIdIdentifier;
        }

        var iriUrlExpression = buildIriUrlExpression({
          idIdentifier: idIdentifier
        });
        return stylesOrIdIdentifier.split(createIriUrl(idValue)).flatMap(function (s, i, splits) {
          return i === splits.length - 1 ? [s] : [s, iriUrlExpression];
        });
      });
    }, [stylesString]).map(function (stylesOrIdIdentifier) {
      return typeof stylesOrIdIdentifier === 'string' ? t.stringLiteral(stylesOrIdIdentifier) : stylesOrIdIdentifier;
    });
  };

  var svgDefsElementsVisitor = {
    JSXOpeningElement: function JSXOpeningElement(path, state) {
      var idAttribute = path.get('attributes').find(function (a) {
        return a.get('name').isJSXIdentifier({
          name: idAttributeName
        });
      });

      if (!idAttribute || !isStringLiteralAttribute(idAttribute)) {
        return;
      }

      var newIdIdentifier = state.idValuesContainer.createIdIdentifier(idAttribute.node.value.value);
      idAttribute.replaceWith(t.jsxAttribute(t.jsxIdentifier(idAttributeName), t.jsxExpressionContainer(newIdIdentifier)));
    }
  };
  var svgDefsVisitor = {
    JSXElement: function JSXElement(path, state) {
      if (isDefsPath(path)) {
        path.traverse(svgDefsElementsVisitor, state);
      }
    }
  };
  var styleTagsVisitor = {
    StringLiteral: function StringLiteral(path, state) {
      var stylesString = path.node.value;
      var iriMatches = new Set(Array.from(stylesString.matchAll(iriGlobalMatchRegex)).map(function (x) {
        return x[1];
      }));
      var idValuesWithIdentifiers = Array.from(iriMatches).map(function (idValue) {
        return [idValue, state.idValuesContainer.getIdIdentifier(idValue)];
      }).filter(function (_ref4) {
        var _ref5 = _slicedToArray(_ref4, 2),
            idIdentifier = _ref5[1];

        return idIdentifier;
      });

      if (idValuesWithIdentifiers.length === 0) {
        return;
      }

      var stylesStringLiterals = splitStylesStringByIriToLiterals(stylesString, idValuesWithIdentifiers);
      var concatenatedStyles = stylesStringLiterals.reduce(function (node, styleLiteral) {
        return node ? t.binaryExpression('+', node, styleLiteral) : styleLiteral;
      });
      path.replaceWith(concatenatedStyles);
    }
  };
  var svgElementsVisitor = {
    JSXElement: function JSXElement(path, state) {
      if (isStylePath(path)) {
        path.traverse(styleTagsVisitor, state);
      }
    },
    JSXOpeningElement: function JSXOpeningElement(path, state) {
      path.get('attributes').forEach(function (attribute) {
        if (!isStringLiteralAttribute(attribute)) {
          return;
        }

        var attributeValue = attribute.node.value.value;
        var iriMatches = attributeValue.match(iriExactMatchRegex);

        if (!iriMatches) {
          return;
        }

        var currentIdValue = iriMatches[1];
        var idIdentifier = state.idValuesContainer.getIdIdentifier(currentIdValue);

        if (idIdentifier) {
          attribute.replaceWith(t.jsxAttribute(t.jsxIdentifier(attribute.node.name.name), t.jsxExpressionContainer(buildIriUrlExpression({
            idIdentifier: idIdentifier
          }))));
        }
      });
    }
  };
  var renderedJsxVisitor = {
    JSXElement: function JSXElement(path, state) {
      if (isSvgPath(path)) {
        path.traverse(svgDefsVisitor, state);
        path.traverse(svgElementsVisitor, state);
      }
    }
  };
  var componentVisitor = {
    Function: function Function(path) {
      var idValuesContainer = createIdValuesContainer(function () {
        return path.scope.generateUidIdentifier(idAttributeName);
      });
      path.traverse(renderedJsxVisitor, {
        idValuesContainer: idValuesContainer
      });

      if (!idValuesContainer.hasIdentifiers()) {
        return;
      } // If component is an arrow function with implicit return, add return statement


      if (path.isArrowFunctionExpression()) {
        path.arrowFunctionToExpression();
      }

      var body = path.get('body');
      var rootPath = path.findParent(function (p) {
        return p.isProgram();
      });
      rootPath.unshiftContainer('body', buildIdGeneratorHookImportStatement());
      idValuesContainer.getIdentifiers().forEach(function (idIdentifier) {
        body.unshiftContainer('body', buildIdIdentifierGeneratorStatement({
          idIdentifier: idIdentifier
        }));
      });
    }
  };
  return {
    inherits: _pluginSyntaxJsx["default"],
    visitor: componentVisitor
  };
};

var _default = plugin;
exports["default"] = _default;